/**
 * @file
 * VuoCompilerModule interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This interface description may be modified and distributed under the terms of the GNU Lesser General Public License (LGPL) version 2 or later.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOCOMPILERMODULE_HH
#define VUOCOMPILERMODULE_HH

#include "VuoBaseDetail.hh"
#include "VuoModule.hh"

#include "VuoCompilerBitcodeParser.hh"

/**
 * A node class or type defined in an LLVM module.
 *
 * This class would be the compiler detail class for @c VuoModule, except that the inheritance doesn't work out.
 */
class VuoCompilerModule
{
private:
	static bool isModule(Module *module);

protected:
	vector<string> dependencies;  ///< The @c moduleDependencies in this module's definition
	Module *module;  ///< The LLVM module that defines this node class or type
	VuoModule *base;  ///< The (pseudo) base for this (pseudo) compiler detail class
	VuoCompilerBitcodeParser *parser;  ///< The parser of the LLVM module

	VuoCompilerModule(VuoModule *base, Module *module);
	virtual ~VuoCompilerModule();

	virtual void parse(void);
	virtual void parseMetadata(void);
	virtual set<string> globalsToRename(void);
	string nameForGlobal(string genericGlobalVarOrFuncName);
	void renameGlobalVarsAndFuncs(void);

	friend class TestVuoCompilerModule;

public:
	static VuoCompilerModule * newModule(string moduleKey, Module *module);

	static Function * declareFunctionInModule(Module *module, Function *function);
	vector<string> getDependencies(void);
	Module * getModule(void);
};

#endif // VUOCOMPILERMODULE_HH
