﻿/**
 * @file
 * VuoCompilerOutputEventPortClass implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code is proprietary, and may not be redistributed.
 * For more information, see http://vuo.org/license.
 */

#include "VuoCompilerOutputEventPortClass.hh"
#include "VuoCompilerOutputEventPort.hh"

#include "VuoPort.hh"

/**
 * Creates a conductor output port type with the specified data type @c type.
 */
VuoCompilerOutputEventPortClass::VuoCompilerOutputEventPortClass(string name, Type *type) :
	VuoCompilerEventPortClass(name, type)
{
}

/**
 * Creates a conductor output port type.
 */
VuoCompilerOutputEventPortClass::VuoCompilerOutputEventPortClass(string name) :
	VuoCompilerEventPortClass(name)
{
}

/**
 * Creates a conductor output port based on this port type, and creates its corresponding base @c VuoPort.
 */
VuoCompilerPort * VuoCompilerOutputEventPortClass::newPort(void)
{
	return new VuoCompilerOutputEventPort(new VuoPort(getBase()));
}

/**
 * Creates a conductor output port based on this port type, using the pre-existing @c port as its base.
 */
VuoCompilerPort * VuoCompilerOutputEventPortClass::newPort(VuoPort *port)
{
	return new VuoCompilerOutputEventPort(port);
}


#pragma mark Getters and setters

/**
 * Returns the data class for this port.  NULL if the port is event-only.
 */
VuoCompilerOutputDataClass * VuoCompilerOutputEventPortClass::getDataClass(void)
{
	return (VuoCompilerOutputDataClass *)VuoCompilerEventPortClass::getDataClass();
}
