CONFIG_OLD = $$CONFIG

# .pro files that specify HEADERS to be included in the framework:
API_HEADER_LISTS = \
	../base/base.pro \
	../compiler/compiler.pro \
	../renderer/renderer.pro \
	../runtime/runtime.pro \
	../type/type.pro \
	../type/list/list.pro

for(header_list, API_HEADER_LISTS) {
	include($${header_list})
	EXPLICIT_HEADERS = $$find(HEADERS, "^[^\\*]+$")  # Exclude header paths containing wildcards
	for(header, EXPLICIT_HEADERS) {
		FRAMEWORK_VUO_HEADERS.files += $$join(header, , $${dirname(header_list)}/, )
		MASTER_VUO_HEADER_LIST += $${header}
	}
	HEADERS = ""
}

FRAMEWORK_VUO_STUB_HEADER = "Vuo.stub.h"
VUO_PRI_LIBS = $$LIBS


# Build the MODULE_OBJECTS list, containing compiled Vuo Modules to be copied to the Framework.
MODULE_LISTS = \
	../node/node.pro \
	../type/type.pro \
	../type/list/list.pro

for(module_list, MODULE_LISTS) {
	NODE_SOURCES = ""
	NODE_LIBRARY_SOURCES = ""
	NODE_LIBRARY_SHARED_SOURCES = ""
	NODE_LIBRARY_SHARED_SOURCES_DEPENDENT_ON_CONTEXT = ""
	TYPE_SOURCES = ""
	TYPE_LIST_SOURCES = ""

	include($${module_list})

	module_list_sources = \
		$$NODE_SOURCES \
		$$TYPE_SOURCES \
		$$TYPE_LIST_SOURCES
	for(sourcefile, module_list_sources) {
		objectfile = $${dirname(module_list)}/$${basename(sourcefile)}
		objectfile ~= s/\\.cc?$/.bc
		MODULE_OBJECTS += $$objectfile
	}

	for(sourcefile, NODE_LIBRARY_SOURCES) {
		objectfile = $${dirname(module_list)}/lib$${basename(sourcefile)}
		objectfile ~= s/\\.cc?$/.bc
		MODULE_OBJECTS += $$objectfile
	}

	SHARED = $$NODE_LIBRARY_SHARED_SOURCES $$NODE_LIBRARY_SHARED_SOURCES_DEPENDENT_ON_CONTEXT
	for(sourcefile, SHARED) {
		objectfile = $${dirname(module_list)}/lib$${basename(sourcefile)}
		objectfile ~= s/\\.cc?$/.dylib
		MODULE_OBJECTS += $$objectfile
		QMAKE_LFLAGS += $$objectfile
		headerfile = $${basename(sourcefile)}
		headerfile ~= s/\\.cc?$/.h
		FRAMEWORK_VUO_HEADERS.files += $${dirname(module_list)}/$$headerfile
		MASTER_VUO_HEADER_LIST += $$headerfile
	}
}


# Now reset variables as appropriate for building Vuo.framework
SOURCES = ""
BASE_STUB_SOURCES = ""
INCLUDEPATH = ""
NODE_SOURCES = ""
NODE_LIBRARY_SOURCES = ""
NODE_LIBRARY_SHARED_SOURCES = ""
NODE_LIBRARY_SHARED_SOURCES_DEPENDENT_ON_CONTEXT = ""
OTHER_FILES = ""
QMAKE_AR_CMD = ""
QMAKE_CLEAN = ""
QMAKE_RANLIB = ""
RUNTIME_C_SOURCES = ""
RUNTIME_CXX_SOURCES = ""
RUNTIME_LOADER_SOURCES = ""
TYPE_SOURCES = ""
TYPE_LIST_SOURCES = ""
TEMPLATE = lib
QMAKE_POST_LINK = ""
LIBS = ""


CONFIG = $$CONFIG_OLD VuoBase VuoCompiler VuoRenderer VuoRuntime json lib_bundle
LIBS += \
	$${ICU_ROOT}/lib/libicuuc.a \
	$${ICU_ROOT}/lib/libicudata.a
TARGET = Vuo

include(../vuo.pri)
CONFIG -= moc

VERSION = $${VUO_VERSION} # used for clang "-current_version" and "-compatibility_version" arguments
QMAKE_FRAMEWORK_VERSION = $${VUO_VERSION} # used for Vuo.framework/Versions/ subdirectory name

QMAKE_LFLAGS += $${QMAKE_LFLAGS_X86_64}
QMAKE_LFLAGS += -Wl,-force_load,$${ROOT}/base/libVuoBase.a
QMAKE_LFLAGS += -Wl,-force_load,$${ROOT}/compiler/libVuoCompiler.a
QMAKE_LFLAGS += -Wl,-force_load,$${ROOT}/renderer/libVuoRenderer.a
QMAKE_LFLAGS += -Wl,-force_load,$${ROOT}/type/libVuoType.a
QMAKE_LFLAGS += -Wl,-force_load,$${ROOT}/type/list/libVuoTypeList.a


# Add the third-party libraries that Vuo nodes/types depend on
MODULE_OBJECTS += \
	$$ICU_ROOT/lib/libicuuc.a \
	$$ICU_ROOT/lib/libicudata.a \
	$$JSONC_ROOT/lib/libjson.a \
	$$MUPARSER_ROOT/lib/libmuparser.a \
	$$FREEIMAGE_ROOT/lib/libfreeimage.a \
	$$FREETYPE_ROOT/lib/libfreetype.a \
	$$CURL_ROOT/lib/libcurl.a \
	$$RTMIDI_ROOT/lib/librtmidi.a \
	$$ASSIMP_ROOT/lib/libassimp.a


# Create and populate Headers directory
HEADERS_DEST_DIR = "Vuo.framework/Versions/$${QMAKE_FRAMEWORK_VERSION}/Headers"
createHeadersDir.commands += rm -rf $${HEADERS_DEST_DIR} &&
createHeadersDir.commands += mkdir -p $${HEADERS_DEST_DIR} &&
createHeadersDir.commands += ln -s . $${HEADERS_DEST_DIR}/Vuo &&
createHeadersDir.commands += cp $$FRAMEWORK_VUO_HEADERS.files $${HEADERS_DEST_DIR} &&
createHeadersDir.commands += ./generateFrameworkHeader.pl $${FRAMEWORK_VUO_STUB_HEADER} $${MASTER_VUO_HEADER_LIST} > $${HEADERS_DEST_DIR}/Vuo.h
createHeadersDir.depends += $$FRAMEWORK_VUO_HEADERS.files $$FRAMEWORK_VUO_STUB_HEADER
createHeadersDir.target = $${HEADERS_DEST_DIR}

POST_TARGETDEPS += $${HEADERS_DEST_DIR}
QMAKE_EXTRA_TARGETS += createHeadersDir


# Create Resources directory; populate it with Info.plist
RESOURCES_DEST_DIR = "Vuo.framework/Versions/$${QMAKE_FRAMEWORK_VERSION}/Resources"
createResourcesDir.commands += rm -rf $${RESOURCES_DEST_DIR} &&
createResourcesDir.commands += mkdir -p $${RESOURCES_DEST_DIR} &&
createResourcesDir.commands += cat Info.plist
createResourcesDir.commands += | sed '"s/@SHORT_VERSION@/$$VUO_VERSION \\(r`svnversion -n` on `date +%Y.%m.%d`\\)/"'
createResourcesDir.commands += > $${RESOURCES_DEST_DIR}/Info.plist
createResourcesDir.depends += Info.plist
createResourcesDir.target = $${RESOURCES_DEST_DIR}

POST_TARGETDEPS += $${RESOURCES_DEST_DIR}
QMAKE_EXTRA_TARGETS += createResourcesDir


# Create and populate Licenses directory
LICENSES_DEST_DIR = "Vuo.framework/Versions/$${QMAKE_FRAMEWORK_VERSION}/Licenses"
createLicensesDir.commands += rm -rf $${LICENSES_DEST_DIR} &&
createLicensesDir.commands += mkdir -p $${LICENSES_DEST_DIR} &&
createLicensesDir.commands += cp ../license/* $${LICENSES_DEST_DIR}
createLicensesDir.depends += ../license/*
createLicensesDir.target = $${LICENSES_DEST_DIR}

POST_TARGETDEPS += $${LICENSES_DEST_DIR}
QMAKE_EXTRA_TARGETS += createLicensesDir


# Create Modules directory
# Populate it with Vuo Node Classes, Types, and Libraries
MODULES_DEST_DIR = "Vuo.framework/Versions/$${QMAKE_FRAMEWORK_VERSION}/Modules"
copyModules.commands += rm -rf $${MODULES_DEST_DIR} &&
copyModules.commands += mkdir -p $${MODULES_DEST_DIR} &&
copyModules.commands += cp $$MODULE_OBJECTS $$MODULES_DEST_DIR
copyModules.depends += $$MODULE_OBJECTS
copyModules.target = $${MODULES_DEST_DIR}
POST_TARGETDEPS += $${MODULES_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyModules


# Create Frameworks directory
FRAMEWORKS_DEST_SUBDIR = "Versions/$${QMAKE_FRAMEWORK_VERSION}/Frameworks"
FRAMEWORKS_DEST_DIR = "Vuo.framework/$$FRAMEWORKS_DEST_SUBDIR"
createFrameworksDir.commands += rm -rf $${FRAMEWORKS_DEST_DIR} &&
createFrameworksDir.commands += mkdir -p $${FRAMEWORKS_DEST_DIR}
createFrameworksDir.target = $${FRAMEWORKS_DEST_DIR}
POST_TARGETDEPS += $${FRAMEWORKS_DEST_DIR}
QMAKE_EXTRA_TARGETS += createFrameworksDir


# Copy VuoRuntime files to the Frameworks directory
VUORUNTIME_DEST_DIR = "$$FRAMEWORKS_DEST_DIR/VuoRuntime.framework"
copyVuoRuntime.commands += rm -Rf $${VUORUNTIME_DEST_DIR} &&
copyVuoRuntime.commands += mkdir -p $${VUORUNTIME_DEST_DIR} &&
copyVuoRuntime.commands += cp $$ROOT/runtime/*\\.bc $$ROOT/runtime/VuoCompositionLoader $${VUORUNTIME_DEST_DIR} &&
copyVuoRuntime.commands += cp $$ROOT/base/VuoCompositionStub.dylib $${VUORUNTIME_DEST_DIR}
copyVuoRuntime.depends += $$ROOT/runtime/*.bc
copyVuoRuntime.depends += $$ROOT/runtime/VuoCompositionLoader
copyVuoRuntime.depends += $$ROOT/base/VuoCompositionStub.dylib
copyVuoRuntime.target = $${VUORUNTIME_DEST_DIR}
POST_TARGETDEPS += $${VUORUNTIME_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyVuoRuntime


# Copy ZeroMQ library to the Frameworks directory
ZEROMQ_LIBS_DEST_DIR = "$$FRAMEWORKS_DEST_DIR/zmq.framework"
copyZeroMQLibs.commands += rm -rf $${ZEROMQ_LIBS_DEST_DIR} &&
copyZeroMQLibs.commands += mkdir -p $${ZEROMQ_LIBS_DEST_DIR} &&
copyZeroMQLibs.commands += cp $${ZMQ_ROOT}/lib/libzmq.a $${ZEROMQ_LIBS_DEST_DIR}
copyZeroMQLibs.target = $${ZEROMQ_LIBS_DEST_DIR}
POST_TARGETDEPS += $${ZEROMQ_LIBS_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyZeroMQLibs


# Copy ZeroMQ headers to the Frameworks directory
ZEROMQ_HEADERS_DEST_DIR = "$$FRAMEWORKS_DEST_DIR/zmq.framework/Headers"
copyZeroMQHeaders.commands += rm -rf $${ZEROMQ_HEADERS_DEST_DIR} &&
copyZeroMQHeaders.commands += mkdir -p $${ZEROMQ_HEADERS_DEST_DIR} &&
copyZeroMQHeaders.commands += cp -r $${ZMQ_ROOT}/include/* $${ZEROMQ_HEADERS_DEST_DIR}
copyZeroMQHeaders.target = $${ZEROMQ_HEADERS_DEST_DIR}
POST_TARGETDEPS += $${ZEROMQ_HEADERS_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyZeroMQHeaders


# Copy JSON-C library to the Frameworks directory
JSON_LIBS_DEST_DIR = "$$FRAMEWORKS_DEST_DIR/json.framework/"
copyJsonLibs.commands += rm -rf $${JSON_LIBS_DEST_DIR} &&
copyJsonLibs.commands += mkdir -p $${JSON_LIBS_DEST_DIR} &&
copyJsonLibs.commands += cp $${JSONC_ROOT}/lib/libjson.dylib $${JSON_LIBS_DEST_DIR} &&
copyJsonLibs.commands += chmod +wx "$$JSON_LIBS_DEST_DIR/libjson.dylib" &&
copyJsonLibs.commands += install_name_tool -id "@rpath/$$JSON_LIBS_DEST_DIR/libjson.dylib" "$$JSON_LIBS_DEST_DIR/libjson.dylib"
copyJsonLibs.target = $${JSON_LIBS_DEST_DIR}
POST_TARGETDEPS += $${JSON_LIBS_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyJsonLibs


# Copy CRuntime files to the Frameworks directory
CRUNTIME_LIBS_DEST_DIR = "$$FRAMEWORKS_DEST_DIR/CRuntime.framework"
copyCRuntime.commands += rm -rf $${CRUNTIME_LIBS_DEST_DIR} &&
copyCRuntime.commands += mkdir -p $${CRUNTIME_LIBS_DEST_DIR} &&
copyCRuntime.commands += cp $${ROOT}/compiler/binary/crt1.o $${CRUNTIME_LIBS_DEST_DIR}/
copyCRuntime.target = $${CRUNTIME_LIBS_DEST_DIR}
POST_TARGETDEPS += $${CRUNTIME_LIBS_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyCRuntime


# Copy QtCore framework to Vuo.framework's Frameworks folder
QtCore.files = $${QT_ROOT}/lib/QtCore.framework
QtCore.path = $$FRAMEWORKS_DEST_SUBDIR
QMAKE_BUNDLE_DATA += QtCore

# Copy QtGui framework to Vuo.framework's Frameworks folder
QtGui.files = $${QT_ROOT}/lib/QtGui.framework
QtGui.path = $$FRAMEWORKS_DEST_SUBDIR
QMAKE_BUNDLE_DATA += QtGui

# Copy QtWidgets framework to Vuo.framework's Frameworks folder
QtWidgets.files = $${QT_ROOT}/lib/QtWidgets.framework
QtWidgets.path = $$FRAMEWORKS_DEST_SUBDIR
QMAKE_BUNDLE_DATA += QtWidgets

# Copy QtMacExtras framework to Vuo.framework's Frameworks folder
QtMacExtras.files = $${QT_ROOT}/lib/QtMacExtras.framework
QtMacExtras.path = $$FRAMEWORKS_DEST_SUBDIR
QMAKE_BUNDLE_DATA += QtMacExtras

# Copy QtPrintSupport framework to Vuo.framework's Frameworks folder
QtPrintSupport.files = $${QT_ROOT}/lib/QtPrintSupport.framework
QtPrintSupport.path = $$FRAMEWORKS_DEST_SUBDIR
QMAKE_BUNDLE_DATA += QtPrintSupport

# Copy QtOpenGL framework to Vuo.framework's Frameworks folder
QtOpenGL.files = $${QT_ROOT}/lib/QtOpenGL.framework
QtOpenGL.path = $$FRAMEWORKS_DEST_SUBDIR
QMAKE_BUNDLE_DATA += QtOpenGL

# Copy QtSvg framework to Vuo.framework's Frameworks folder
QtSvg.files = $${QT_ROOT}/lib/QtSvg.framework
QtSvg.path = $$FRAMEWORKS_DEST_SUBDIR
QMAKE_BUNDLE_DATA += QtSvg

# Copy QtXml framework to Vuo.framework's Frameworks folder
QtXml.files = $${QT_ROOT}/lib/QtXml.framework
QtXml.path = $$FRAMEWORKS_DEST_SUBDIR
QMAKE_BUNDLE_DATA += QtXml

# Copy Qt plugins to Vuo.framework's Frameworks folder
QTPLUGINS_DEST_DIR = "$$FRAMEWORKS_DEST_DIR/QtPlugins"
copyQtPlugins.commands += rm -rf $$QTPLUGINS_DEST_DIR &&
copyQtPlugins.commands += mkdir -p $$QTPLUGINS_DEST_DIR &&
copyQtPlugins.commands += cp -R $$QT_ROOT/plugins/* $$QTPLUGINS_DEST_DIR/
copyQtPlugins.target = $$QTPLUGINS_DEST_DIR
PRE_TARGETDEPS += $$QTPLUGINS_DEST_DIR
QMAKE_EXTRA_TARGETS += copyQtPlugins


# Remove unneeded files from the Qt frameworks
QMAKE_PRE_LINK += rm -Rf "$$FRAMEWORKS_DEST_DIR/Qt*.framework/Contents" &&
QMAKE_PRE_LINK += rm -f "$$FRAMEWORKS_DEST_DIR/Qt*.framework/Qt*.prl" &&
QMAKE_PRE_LINK += rm -f "$$FRAMEWORKS_DEST_DIR/Qt*.framework/Qt*_debug" &&
QMAKE_PRE_LINK += rm -f "$$FRAMEWORKS_DEST_DIR/QtPlugins/*/*_debug.dylib" &&
QMAKE_PRE_LINK += rm -f "$$FRAMEWORKS_DEST_DIR/Qt*.framework/Versions/$$QT_MAJOR_VERSION/Qt*_debug" &&

# Enable Vuo.framework to specify where the linker should look for the Qt frameworks
QMAKE_PRE_LINK += chmod +wx "$$FRAMEWORKS_DEST_DIR/QtCore.framework/Versions/$$QT_MAJOR_VERSION/QtCore" &&
QMAKE_PRE_LINK += install_name_tool -id "@rpath/$$FRAMEWORKS_DEST_DIR/QtCore.framework/QtCore" "$$FRAMEWORKS_DEST_DIR/QtCore.framework/QtCore" &&
QMAKE_PRE_LINK += chmod +wx "$$FRAMEWORKS_DEST_DIR/QtGui.framework/Versions/$$QT_MAJOR_VERSION/QtGui" &&
QMAKE_PRE_LINK += install_name_tool -id "@rpath/$$FRAMEWORKS_DEST_DIR/QtGui.framework/QtGui" "$$FRAMEWORKS_DEST_DIR/QtGui.framework/QtGui" &&
QMAKE_PRE_LINK += chmod +wx "$$FRAMEWORKS_DEST_DIR/QtWidgets.framework/Versions/$$QT_MAJOR_VERSION/QtWidgets" &&
QMAKE_PRE_LINK += install_name_tool -id "@rpath/$$FRAMEWORKS_DEST_DIR/QtWidgets.framework/QtWidgets" "$$FRAMEWORKS_DEST_DIR/QtWidgets.framework/QtWidgets" &&
QMAKE_PRE_LINK += chmod +wx "$$FRAMEWORKS_DEST_DIR/QtMacExtras.framework/Versions/$$QT_MAJOR_VERSION/QtMacExtras" &&
QMAKE_PRE_LINK += install_name_tool -id "@rpath/$$FRAMEWORKS_DEST_DIR/QtMacExtras.framework/QtMacExtras" "$$FRAMEWORKS_DEST_DIR/QtMacExtras.framework/QtMacExtras" &&
QMAKE_PRE_LINK += chmod +wx "$$FRAMEWORKS_DEST_DIR/QtPrintSupport.framework/Versions/$$QT_MAJOR_VERSION/QtPrintSupport" &&
QMAKE_PRE_LINK += install_name_tool -id "@rpath/$$FRAMEWORKS_DEST_DIR/QtPrintSupport.framework/QtPrintSupport" "$$FRAMEWORKS_DEST_DIR/QtPrintSupport.framework/QtPrintSupport" &&
QMAKE_PRE_LINK += chmod +wx "$$FRAMEWORKS_DEST_DIR/QtOpenGL.framework/Versions/$$QT_MAJOR_VERSION/QtOpenGL" &&
QMAKE_PRE_LINK += install_name_tool -id "@rpath/$$FRAMEWORKS_DEST_DIR/QtOpenGL.framework/QtOpenGL" "$$FRAMEWORKS_DEST_DIR/QtOpenGL.framework/QtOpenGL" &&
QMAKE_PRE_LINK += chmod +wx "$$FRAMEWORKS_DEST_DIR/QtSvg.framework/Versions/$$QT_MAJOR_VERSION/QtSvg" &&
QMAKE_PRE_LINK += install_name_tool -id "@rpath/$$FRAMEWORKS_DEST_DIR/QtSvg.framework/QtSvg" "$$FRAMEWORKS_DEST_DIR/QtSvg.framework/QtSvg" &&
QMAKE_PRE_LINK += chmod +wx "$$FRAMEWORKS_DEST_DIR/QtXml.framework/Versions/$$QT_MAJOR_VERSION/QtXml" &&
QMAKE_PRE_LINK += install_name_tool -id "@rpath/$$FRAMEWORKS_DEST_DIR/QtXml.framework/QtXml" "$$FRAMEWORKS_DEST_DIR/QtXml.framework/QtXml" &&
QMAKE_PRE_LINK += ./fixRpaths.sh $$QT_ROOT $$FRAMEWORKS_DEST_DIR $$QT_MAJOR_VERSION &&
QMAKE_PRE_LINK += ./fixRpaths.sh /usr/local $$FRAMEWORKS_DEST_DIR $$QT_MAJOR_VERSION &&

# Link first with our install_name_tool'd frameworks when producing Vuo.framework/Vuo
LIBS += \
	-F $$FRAMEWORKS_DEST_DIR \
	-framework QtCore \
	-framework QtGui \
	-framework QtWidgets \
	-framework QtMacExtras \
	-framework QtPrintSupport
# Then bring in the rest of the LIBS from vuo.pri
LIBS += $$VUO_PRI_LIBS
LIBS += \
	-framework OpenGL \
	-framework IOSurface \
	$$ROOT/node/VuoImageRenderer.o


# Copy Clang headers to the Frameworks directory
CLANG_HEADERS_DEST_DIR = "$$FRAMEWORKS_DEST_DIR/clang.framework/Headers"
copyClangHeaders.commands += rm -rf $${CLANG_HEADERS_DEST_DIR} &&
copyClangHeaders.commands += mkdir -p $${CLANG_HEADERS_DEST_DIR} &&
copyClangHeaders.commands += cp -r $${LLVM_ROOT}/include/clang/* $${CLANG_HEADERS_DEST_DIR}
copyClangHeaders.target = $${CLANG_HEADERS_DEST_DIR}
POST_TARGETDEPS += $${CLANG_HEADERS_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyClangHeaders


# Copy LLVM headers to the Frameworks directory
LLVM_HEADERS_DEST_DIR = "$$FRAMEWORKS_DEST_DIR/llvm.framework/Headers"
copyLLVMHeaders.commands += rm -rf $${LLVM_HEADERS_DEST_DIR} &&
copyLLVMHeaders.commands += mkdir -p $${LLVM_HEADERS_DEST_DIR} &&
copyLLVMHeaders.commands += cp -r $${LLVM_ROOT}/include/llvm/* $${LLVM_HEADERS_DEST_DIR}
copyLLVMHeaders.target = $${LLVM_HEADERS_DEST_DIR}
POST_TARGETDEPS += $${LLVM_HEADERS_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyLLVMHeaders


# Copy Graphviz libraries to the Frameworks directory
GRAPHVIZ_LIBS_DEST_DIR = "$$FRAMEWORKS_DEST_DIR/graphviz.framework"
copyGraphvizLibs.commands += rm -rf "$${GRAPHVIZ_LIBS_DEST_DIR}" &&
copyGraphvizLibs.commands += mkdir -p "$${GRAPHVIZ_LIBS_DEST_DIR}" &&
copyGraphvizLibs.commands += mkdir -p "$${GRAPHVIZ_LIBS_DEST_DIR}/graphviz" &&
copyGraphvizLibs.commands += cp "$${GRAPHVIZ_ROOT}/lib/graphviz/libgvplugin_dot_layout.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}" &&
copyGraphvizLibs.commands += ln -s "libgvplugin_dot_layout.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_dot_layout.dylib" &&
copyGraphvizLibs.commands += cp "$${GRAPHVIZ_ROOT}/lib/graphviz/libgvplugin_core.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}" &&
copyGraphvizLibs.commands += ln -s "libgvplugin_core.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_core.dylib" &&
copyGraphvizLibs.commands += cp "$${GRAPHVIZ_ROOT}/lib/libcdt.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}" &&
copyGraphvizLibs.commands += ln -s "libcdt.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libcdt.dylib" &&
copyGraphvizLibs.commands += cp "$${GRAPHVIZ_ROOT}/lib/libgraph.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}" &&
copyGraphvizLibs.commands += ln -s "libgraph.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgraph.dylib" &&
copyGraphvizLibs.commands += cp "$${GRAPHVIZ_ROOT}/lib/libgvc.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}" &&
copyGraphvizLibs.commands += ln -s "libgvc.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.dylib" &&
copyGraphvizLibs.commands += cp "$${GRAPHVIZ_ROOT}/lib/libpathplan.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}" &&
copyGraphvizLibs.commands += ln -s "libpathplan.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libpathplan.dylib" &&
copyGraphvizLibs.commands += cp "$${GRAPHVIZ_ROOT}/lib/libxdot.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}" &&
copyGraphvizLibs.commands += ln -s "libxdot.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libxdot.dylib"
copyGraphvizLibs.target = $${GRAPHVIZ_LIBS_DEST_DIR}
POST_TARGETDEPS += $${GRAPHVIZ_LIBS_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyGraphvizLibs

# Adjust where Graphviz libraries search for each other
QMAKE_PRE_LINK += chmod +w "$${GRAPHVIZ_LIBS_DEST_DIR}/*.dylib" &&
QMAKE_PRE_LINK += install_name_tool -id "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_core.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_core.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libcdt.5.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libcdt.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_core.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libgraph.5.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgraph.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_core.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libgvc.6.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_core.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libpathplan.4.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libpathplan.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_core.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libxdot.4.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libxdot.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_core.6.dylib" &&

QMAKE_PRE_LINK += install_name_tool -id "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_dot_layout.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_dot_layout.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libcdt.5.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libcdt.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_dot_layout.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libgraph.5.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgraph.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_dot_layout.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libgvc.6.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_dot_layout.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libpathplan.4.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libpathplan.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_dot_layout.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libxdot.4.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libxdot.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_dot_layout.6.dylib" &&

QMAKE_PRE_LINK += install_name_tool -id "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libcdt.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libcdt.5.dylib" &&

QMAKE_PRE_LINK += install_name_tool -id "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgraph.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgraph.5.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libcdt.5.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libcdt.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgraph.5.dylib" &&

QMAKE_PRE_LINK += install_name_tool -id "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.6.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libcdt.5.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libcdt.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libgraph.5.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgraph.5.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libpathplan.4.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libpathplan.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.6.dylib" &&
QMAKE_PRE_LINK += install_name_tool -change "$${GRAPHVIZ_ROOT}/lib/libxdot.4.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libxdot.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.6.dylib" &&

QMAKE_PRE_LINK += install_name_tool -id "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libpathplan.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libpathplan.4.dylib" &&

QMAKE_PRE_LINK += install_name_tool -id "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libxdot.4.dylib" "$${GRAPHVIZ_LIBS_DEST_DIR}/libxdot.4.dylib" &&

QMAKE_PRE_LINK += chmod -w "$${GRAPHVIZ_LIBS_DEST_DIR}/*.dylib" &&


# Create MacOS directory
MACOS_DEST_DIR = "Vuo.framework/Versions/$${QMAKE_FRAMEWORK_VERSION}/MacOS"
createMacOSDir.commands += rm -rf $${MACOS_DEST_DIR} &&
createMacOSDir.commands += mkdir -p $${MACOS_DEST_DIR}
createMacOSDir.target = $${MACOS_DEST_DIR}
POST_TARGETDEPS += $${MACOS_DEST_DIR}
QMAKE_EXTRA_TARGETS += createMacOSDir


# Copy Clang and ld64 binaries to the MacOS directory
CLANG_BINS_DEST_DIR = "Vuo.framework/Versions/$${QMAKE_FRAMEWORK_VERSION}/MacOS/Clang/bin"
copyClangBins.commands += rm -rf $${CLANG_BINS_DEST_DIR} &&
copyClangBins.commands += mkdir -p $${CLANG_BINS_DEST_DIR} &&
copyClangBins.commands += cp -r $${LLVM_ROOT}/bin/clang $${CLANG_BINS_DEST_DIR} &&
copyClangBins.commands += cp -r $${LLVM_ROOT}/bin/llvm-link $${CLANG_BINS_DEST_DIR} &&
copyClangBins.commands += ln -s clang $${CLANG_BINS_DEST_DIR}/clang++ &&
copyClangBins.commands += cp -r $${ROOT}/compiler/binary/ld $${CLANG_BINS_DEST_DIR}
copyClangBins.target = $${CLANG_BINS_DEST_DIR}
POST_TARGETDEPS += $${CLANG_BINS_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyClangBins


# Copy Clang system headers to the MacOS directory
CLANG_SYS_HEADERS_DEST_DIR = "Vuo.framework/Versions/$${QMAKE_FRAMEWORK_VERSION}/MacOS/Clang/lib/clang/3.2/include"
copyClangSysHeaders.commands += rm -rf $${CLANG_SYS_HEADERS_DEST_DIR} &&
copyClangSysHeaders.commands += mkdir -p $${CLANG_SYS_HEADERS_DEST_DIR} &&
copyClangSysHeaders.commands += cp -r $${LLVM_ROOT}/lib/clang/3.2/include/* $${CLANG_SYS_HEADERS_DEST_DIR}
copyClangSysHeaders.target = $${CLANG_SYS_HEADERS_DEST_DIR}
POST_TARGETDEPS += $${CLANG_SYS_HEADERS_DEST_DIR}
QMAKE_EXTRA_TARGETS += copyClangSysHeaders

createLinks.commands += ln -sf Versions/Current/Headers Vuo.framework/ &&
createLinks.commands += ln -sf Versions/Current/Resources Vuo.framework/ &&
createLinks.commands += ln -sf Versions/Current/Licenses Vuo.framework/ &&
createLinks.commands += ln -sf Versions/Current/Modules Vuo.framework/ &&
createLinks.commands += ln -sf Versions/Current/Frameworks Vuo.framework/ &&
createLinks.commands += ln -sf Versions/Current/MacOS Vuo.framework/
createLinks.target = Vuo.framework/Headers
POST_TARGETDEPS += Vuo.framework/Headers
QMAKE_EXTRA_TARGETS += createLinks


# Replace symbolic link "Vuo -> Versions/$${QMAKE_FRAMEWORK_VERSION}/Vuo"
# with "Vuo -> Versions/Current/Vuo"
QMAKE_POST_LINK += rm Vuo.framework/Vuo &&
QMAKE_POST_LINK += ln -s Versions/Current/Vuo Vuo.framework/Vuo &&

# Enable applications using the framework to specify where the linker should look for it
VUO_LIB_PATH = Vuo.framework/Versions/$${QMAKE_FRAMEWORK_VERSION}/Vuo
QMAKE_POST_LINK += install_name_tool -id @rpath/$$VUO_LIB_PATH $$VUO_LIB_PATH &&


# Adjust where the Vuo library searches for the Graphviz library
QMAKE_POST_LINK += install_name_tool -change "/usr/local/lib/graphviz/libgvplugin_core.6.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_core.6.dylib" "$${VUO_LIB_PATH}" &&
QMAKE_POST_LINK += install_name_tool -change "/usr/local/lib/graphviz/libgvplugin_dot_layout.6.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgvplugin_dot_layout.6.dylib" "$${VUO_LIB_PATH}" &&
QMAKE_POST_LINK += install_name_tool -change "/usr/local/lib/libcdt.5.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libcdt.5.dylib" "$${VUO_LIB_PATH}" &&
QMAKE_POST_LINK += install_name_tool -change "/usr/local/lib/libgraph.5.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgraph.5.dylib" "$${VUO_LIB_PATH}" &&
QMAKE_POST_LINK += install_name_tool -change "/usr/local/lib/libgvc.6.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libgvc.6.dylib" "$${VUO_LIB_PATH}" &&
QMAKE_POST_LINK += install_name_tool -change "/usr/local/lib/libpathplan.4.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libpathplan.4.dylib" "$${VUO_LIB_PATH}" &&
QMAKE_POST_LINK += install_name_tool -change "/usr/local/lib/libxdot.4.dylib" "@rpath/$${GRAPHVIZ_LIBS_DEST_DIR}/libxdot.4.dylib" "$${VUO_LIB_PATH}" &&


# Some other project files depend on Vuo.framework.
# If qmake was run before Vuo.framework was created, the other projects don't calculate dependencies properly.
# As a litmus test, we know that vuo-link should depend on Vuo.framework's Vuo.h, so re-run qmake if it doesn't.
QMAKE_POST_LINK += ( \
	grep -q framework/Vuo.framework/Versions/$${VUO_VERSION}/Headers/Vuo.h ../compiler/vuo-link/Makefile \
	|| (cd .. && /usr/local/bin/qmake -spec macx-clang CONFIG+=x86_64 -r) \
	) &&


# Force other projects dependent on the framework to rebuild their precompiled headers.
QMAKE_POST_LINK += rm -Rf $$ROOT/editor/VuoEditorApp/pch &&


# Tell qmake not to mess with Info.plist
QMAKE_INFO_PLIST_OUT = /dev/null

QMAKE_CLEAN += "Vuo\\.framework"

# Terminate the series of "&&".
QMAKE_PRE_LINK += true
QMAKE_POST_LINK += true
