/**
 * @file
 * VuoWindowApplication implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "VuoWindowApplication.hh"

#ifdef MAC
#include <objc/runtime.h>
#endif

/**
 * Creates an application and starts the event loop.
 */
VuoWindowApplication::VuoWindowApplication(int argc, char **argv)
	: QApplication(argc, argv)
{
#ifdef MAC
	// Workaround for a bug in Qt 5.1.0-beta1 (https://b33p.net/kosada/node/4905).
	// [NSAutoreleasePool new];
	Class poolClass = (Class)objc_getClass("NSAutoreleasePool");
	SEL newSEL = sel_registerName("new");
	Method poolNewMethod = class_getClassMethod(poolClass, newSEL);
	IMP poolNew = method_getImplementation(poolNewMethod);
	poolNew((id)poolClass, method_getName(poolNewMethod));
#endif

	eventLoop = new QEventLoop;
	timer = dispatch_source_create(DISPATCH_SOURCE_TYPE_TIMER,0,0,dispatch_get_main_queue());
	dispatch_source_set_timer(timer, dispatch_walltime(NULL,0), NSEC_PER_SEC/100, NSEC_PER_SEC/100);
	dispatch_source_set_event_handler(timer, ^{ eventLoop->processEvents(QEventLoop::AllEvents,10); });

	timerCanceled = dispatch_semaphore_create(0);
	dispatch_source_set_cancel_handler(timer, ^{ dispatch_semaphore_signal(timerCanceled); });

	dispatch_resume(timer);
}

/**
 * Stops the event loop and destroys the application.
 */
VuoWindowApplication::~VuoWindowApplication(void)
{
	dispatch_source_cancel(timer);
	dispatch_release(timer);
	dispatch_semaphore_wait(timerCanceled, DISPATCH_TIME_FOREVER);
	dispatch_release(timerCanceled);

	dispatch_sync(dispatch_get_main_queue(), ^{
					  delete eventLoop;
					  delete QApplication::instance();
				  });
}
