/**
 * @file
 * VuoWindowOpenGLInternal interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOWINDOWOPENGLINTERNAL_HH
#define VUOWINDOWOPENGLINTERNAL_HH

#include <QWindow>
#include <QOpenGLContext>

#include "node.h"
#include "VuoGlContext.h"

/**
 * Private implementation of @c VuoWindowOpenGL.
 */
class VuoWindowOpenGLInternal : public QWindow
{
	Q_OBJECT

private:
	void ensureGLContext(void);
	void render(void);

	bool callbacksEnabled;
	void (*initCallback)(void *);
	void (*resizeCallback)(void *, unsigned int width, unsigned int height);
	void (*drawCallback)(void *);
	void *drawContext;

	void (*frameRequested)(VuoInteger); /// @todo change to struct...
	void (*mouseMoved)(VuoPoint2d); /// @todo implement
	void (*mouseLeftPressed)(VuoPoint2d); /// @todo implement

	VuoGlContext vuoGlContext;
	QOpenGLContext *glContext;

	bool actualAspectDiffersSignificantlyFromIdeal(void);
	float aspect;
	bool programmaticallyResizingWindow;
	bool userResizedWindow;

public:
	VuoWindowOpenGLInternal
	(
			void (*initCallback)(void *),
			void (*resizeCallback)(void *, unsigned int width, unsigned int height),
			void (*drawCallback)(void *),
			void *drawContext
	);
	~VuoWindowOpenGLInternal();
	void enableTriggers
	(
			VuoOutputTrigger(frameRequested, VuoInteger), /// @todo change to struct...
			VuoOutputTrigger(mouseMoved, VuoPoint2d),
			VuoOutputTrigger(mouseLeftPressed, VuoPoint2d)
	);
	void disableTriggers(void);
	void scheduleRedraw(void);
	void setAspectRatio(unsigned int pixelsWide, unsigned int pixelsHigh);

protected:
	bool event(QEvent *event);
	void exposeEvent(QExposeEvent *event);
	void resizeEvent(QResizeEvent *event);
};

#endif // VUOWINDOWOPENGLINTERNAL_HH
