/**
 * @file
 * VuoWindowTextInternal interface.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOWINDOWTEXTINTERNAL_HH
#define VUOWINDOWTEXTINTERNAL_HH

#include <QMainWindow>
#include <QCloseEvent>
#include <QPlainTextEdit>

#include "node.h"

/**
 * Private implementation of @c VuoWindowText.
 */
class VuoWindowTextInternal : public QMainWindow
{
	Q_OBJECT

private:
	QPlainTextEdit *pte;
	QString lastPlainTextSet;
	void (* typedLine)(VuoText);
	void (* typedWord)(VuoText);
	void (* typedCharacter)(VuoText);

public:
	VuoWindowTextInternal(void);
	void enableTriggers
	(
			VuoOutputTrigger(typedLine, VuoText),
			VuoOutputTrigger(typedWord, VuoText),
			VuoOutputTrigger(typedCharacter, VuoText)
	);
	void disableTriggers(void);
	void appendLine(const char *text);

private slots:
	void fireTextChangedEvents(void);
	void moveCursorToEnd(void);

protected:
	void closeEvent(QCloseEvent *event);
};

#endif // VUOWINDOWTEXTINTERNAL_HH
