﻿/**
 * @file
 * vuo.console.window node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include <stdlib.h>
#include <unistd.h>
#include <dispatch/dispatch.h>
#include <errno.h>

#include "VuoWindow.h"


const char *moduleName = "Display Console Window";
const char *moduleDescription = "Opens a window that displays console output (text written to the `lineToWrite` input) and accepts text entered by the user (made available on the `typed*` triggers).";
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = true;
const char *moduleKeywords[] = { "text", "string", "print", "log", "write", "read", "input", "output" };
/// @todo remove Qt dependencies after https://b33p.net/kosada/node/4018 is fixed
const char *moduleDependencies[] = {
	"VuoWindow.bc",
	"VuoWindowApplication.bc",
	"VuoWindowTextInternal.bc",
	"VuoWindowOpenGLInternal.bc",
	"moc_VuoWindowApplication.bc",
	"moc_VuoWindowTextInternal.bc",
	"moc_VuoWindowOpenGLInternal.bc",
	"QtCore.framework",
	"QtGui.framework",
	"QtWidgets.framework",
	"objc",
	"z",
	"CoreFoundation.framework",
	"AppKit.framework",
	"Carbon.framework",
	"Security.framework"
};

struct nodeInstanceData
{
	VuoWindowText *window;
};

struct nodeInstanceData * nodeInstanceInit(void)
{
	struct nodeInstanceData *context = (struct nodeInstanceData *)calloc(1,sizeof(struct nodeInstanceData));
	context->window = VuoWindowTextCreate();
	vuoRegister(context, free);
	return context;
}

void nodeInstanceCallbackStart
(
		VuoInstanceData(struct nodeInstanceData *) context,
		VuoOutputTrigger(typedLine, VuoText),
		VuoOutputTrigger(typedWord, VuoText),
		VuoOutputTrigger(typedCharacter, VuoText)
)
{
	VuoWindowTextEnableTriggers((*context)->window, typedLine, typedWord, typedCharacter);
}

void nodeInstanceEvent
(
		VuoInstanceData(struct nodeInstanceData *) context,
		VuoInputData(VuoText, {"default":""}) lineToWrite,
		VuoInputConductor(VuoPortAlwaysConducts, lineToWrite) lineToWriteEvent
)
{
	if (lineToWriteEvent)
		VuoWindowTextAppendLine((*context)->window, lineToWrite);
}

void nodeInstanceCallbackStop
(
		VuoInstanceData(struct nodeInstanceData *) context
)
{
	VuoWindowTextDisableTriggers((*context)->window);
}

void nodeInstanceFini
(
		VuoInstanceData(struct nodeInstanceData *) context
)
{
	VuoWindowTextDestroy((*context)->window);
}
