﻿/**
 * @file
 * vuo.hold.sceneObject node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

const char *moduleName = "Hold Value";
const char *moduleDescription = "This node holds onto the most recent value received through the 'newValue' port and outputs it when an event hits the refresh port. If the 'newValue' port has never received an event, this node instead outputs 'initialValue'.";
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = false;
const char *moduleKeywords[] = { "store", "sample", "feedback" };

bool * nodeInstanceInit()
{
	bool *hasNewValueReceivedEvent = (bool *) malloc(sizeof(bool));
	vuoRegister(hasNewValueReceivedEvent, free);
	*hasNewValueReceivedEvent = false;
	return hasNewValueReceivedEvent;
}

void nodeInstanceEvent
(
		VuoInstanceData(bool *) hasNewValueReceivedEvent,
		VuoInputData(VuoSceneObject) initialValue,
		VuoInputConductor(VuoPortNeverConducts,initialValue) initialValueEvent,
		VuoInputData(VuoSceneObject) newValue,
		VuoInputConductor(VuoPortNeverConducts,newValue) newValueEvent,
		VuoOutputData(VuoSceneObject) heldValue
)
{
	if (newValueEvent)
		**hasNewValueReceivedEvent = true;

	*heldValue = (**hasNewValueReceivedEvent ? newValue : initialValue);
}

void nodeInstanceFini(VuoInstanceData(bool *) hasNewValueReceivedEvent)
{
}
