/**
 * @file
 * vuo.math.areEqual.real node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

const char *moduleName= "Are Equal";
const char *moduleDescription = "Outputs 'true' if all terms are within 'tolerance' of each other (or there are no terms).";
const char *moduleKeywords[] = {"equals", "equality", "true", "false", "compare"};
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = false;

void nodeEvent
(
		VuoInputData(VuoList_VuoInteger) terms,
		VuoInputData(VuoReal, {"default":0.01}) tolerance,
		VuoOutputData(VuoBoolean) equal
)
{
	unsigned long termsCount = VuoListGetCount_VuoReal(terms);
	if (termsCount > 1)
	{
		VuoReal min, max;
		min = max = VuoListGetValueAtIndex_VuoReal(terms, 0);
		for (unsigned long i = 1; i < termsCount; ++i)
		{
			VuoReal term = VuoListGetValueAtIndex_VuoReal(terms, i);
			if (term < min)
				min = term;
			if (term > max)
				max = term;
		}
		*equal = (max - min <= tolerance);
	}
	else
	{
		*equal = true;
	}
}
