﻿/**
 * @file
 * vuo.math.count.integer node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include <stdlib.h>

const char *moduleName = "Count";
const char *moduleDescription = "Keeps track of a count that starts at 0. The count increases by `increment` when that port is pushed. It decreases by `decrement` when that port is pushed. When `refresh` is pushed, the count stays the same.";
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = false;


VuoInteger * nodeInstanceInit()
{
	VuoInteger *countState = (VuoInteger *) malloc(sizeof(VuoInteger));
	vuoRegister(countState, free);
	*countState = 0;
	return countState;
}

void nodeInstanceEvent
(
		VuoInstanceData(VuoInteger *) countState,
		VuoInputData(VuoInteger, {"default":1}) increment,
		VuoInputConductor(VuoPortAlwaysConducts,increment) incrementEvent,
		VuoInputData(VuoInteger, {"default":1}) decrement,
		VuoInputConductor(VuoPortAlwaysConducts,decrement) decrementEvent,
		VuoOutputData(VuoInteger) count
)
{
	if (incrementEvent)
		**countState += increment;
	if (decrementEvent)
		**countState -= decrement;
	*count = **countState;
}


void nodeInstanceFini(VuoInstanceData(VuoInteger *) countState)
{
}
