/**
 *@file
 * vuo.math.gradientNoise.3d node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "VuoGradientNoiseCommon.h"

const char *moduleName = "Make Gradient Noise 3D";
const char *moduleDescription = "Given a 3D position, this node outputs a gradient noise value, using either Ken Perlin's Improved Noise or Simplex algorithm.";
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = false;
const char *moduleKeywords[] = { "perlin", "simplex" };
const char *moduleDependencies[] = { "VuoGradientNoiseCommon.bc" };


void nodeEvent
(
	VuoInputData(VuoPoint3d, {"default":{"x":0,"y":0,"z":0}}) position,
	VuoInputData(VuoGradientNoise, {"default":"perlin"}) gradientNoise,
	VuoOutputData(VuoReal) value
)
{
	if (gradientNoise == VuoGradientNoise_Perlin)
		*value = VuoGradientNoise_perlin3D(position.x, position.y, position.z);
	else if (gradientNoise == VuoGradientNoise_Simplex)
		*value = VuoGradientNoise_simplex3D(position.x, position.y, position.z);
}
