/**
 * @file
 * vuo.midi.filter.controller node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

const char *moduleName = "Filter MIDI Controller";
const char *moduleDescription = "Blocks the `controller` event unless the controller matches all conditions.";
const char *moduleKeywords[] = { "CC", "custom controller" };
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = false;

void nodeEvent
(
		VuoInputData(VuoMIDIController) controller,
		VuoInputConductor(VuoPortMayConduct, controller) controllerEvent,

		VuoInputData(VuoInteger, {"default":1, "suggestedMin":1, "suggestedMax":16}) channelMin,
		VuoInputConductor(VuoPortNeverConducts, channelMin) channelMinEvent,
		VuoInputData(VuoInteger, {"default":16, "suggestedMin":1, "suggestedMax":16}) channelMax,
		VuoInputConductor(VuoPortNeverConducts, channelMax) channelMaxEvent,

		VuoInputData(VuoInteger, {"default":0, "suggestedMin":0, "suggestedMax":127}) controllerNumberMin,
		VuoInputConductor(VuoPortNeverConducts, controllerNumberMin) controllerNumberMinEvent,
		VuoInputData(VuoInteger, {"default":127, "suggestedMin":0, "suggestedMax":127}) controllerNumberMax,
		VuoInputConductor(VuoPortNeverConducts, controllerNumberMax) controllerNumberMaxEvent,

		VuoInputData(VuoInteger, {"default":0, "suggestedMin":0, "suggestedMax":127}) valueMin,
		VuoInputConductor(VuoPortNeverConducts, valueMin) valueMinEvent,
		VuoInputData(VuoInteger, {"default":127, "suggestedMin":0, "suggestedMax":127}) valueMax,
		VuoInputConductor(VuoPortNeverConducts, valueMax) valueMaxEvent,

		VuoOutputData(VuoMIDIController) filteredController,
		VuoOutputConductor(filteredController) filteredControllerEvent
)
{
	if (controller.channel < channelMin || controller.channel > channelMax)
		return;

	if (controller.controllerNumber < controllerNumberMin || controller.controllerNumber > controllerNumberMax)
		return;

	if (controller.value < valueMin || controller.value > valueMax)
		return;

	*filteredController = controller;
	*filteredControllerEvent = true;
}
