/**
 * @file
 * vuo.midi.filter.note node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

const char *moduleName = "Filter MIDI Note";
const char *moduleDescription = "Blocks the `note` event unless the note matches all conditions.";
const char *moduleKeywords[] = { };
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = false;

void nodeEvent
(
		VuoInputData(VuoMIDINote) note,
		VuoInputConductor(VuoPortMayConduct, note) noteEvent,

		VuoInputData(VuoBoolean, {"default":true}) includeNoteOn,
		VuoInputConductor(VuoPortNeverConducts, includeNoteOn) includeNoteOnEvent,

		VuoInputData(VuoBoolean, {"default":false}) includeNoteOff,
		VuoInputConductor(VuoPortNeverConducts, includeNoteOff) includeNoteOffEvent,

		VuoInputData(VuoInteger, {"default":1, "suggestedMin":1, "suggestedMax":16}) channelMin,
		VuoInputConductor(VuoPortNeverConducts, channelMin) channelMinEvent,
		VuoInputData(VuoInteger, {"default":16, "suggestedMin":1, "suggestedMax":16}) channelMax,
		VuoInputConductor(VuoPortNeverConducts, channelMax) channelMaxEvent,

		VuoInputData(VuoInteger, {"default":0, "suggestedMin":0, "suggestedMax":127}) velocityMin,
		VuoInputConductor(VuoPortNeverConducts, velocityMin) velocityMinEvent,
		VuoInputData(VuoInteger, {"default":127, "suggestedMin":0, "suggestedMax":127}) velocityMax,
		VuoInputConductor(VuoPortNeverConducts, velocityMax) velocityMaxEvent,

		VuoInputData(VuoInteger, {"default":0, "suggestedMin":0, "suggestedMax":127}) noteNumberMin,
		VuoInputConductor(VuoPortNeverConducts, noteNumberMin) noteNumberMinEvent,
		VuoInputData(VuoInteger, {"default":127, "suggestedMin":0, "suggestedMax":127}) noteNumberMax,
		VuoInputConductor(VuoPortNeverConducts, noteNumberMax) noteNumberMaxEvent,

		VuoOutputData(VuoMIDINote) filteredNote,
		VuoOutputConductor(filteredNote) filteredNoteEvent
)
{
	if (note.isNoteOn && !includeNoteOn)
		return;
	if (!note.isNoteOn && !includeNoteOff)
		return;

	if (note.channel < channelMin || note.channel > channelMax)
		return;

	if (note.velocity < velocityMin || note.velocity > velocityMax)
		return;

	if (note.noteNumber < noteNumberMin || note.noteNumber > noteNumberMax)
		return;

	*filteredNote = note;
	*filteredNoteEvent = true;
}
