/**
 * @file
 * vuo.midi.receive node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "VuoMIDI.h"

const char *moduleName = "Receive MIDI Events";
const char *moduleDescription = "Listens for MIDI data.  When data is received, this node fires an event for the corresponding type of MIDI data.  The first available MIDI Input device is used, unless a specific `device` is provided.";
const char *moduleKeywords[] = { };
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = true;
/// @todo when https://b33p.net/kosada/node/4018 is fixed, move the VuoMIDI dependencies into VuoMIDI.cc.
const char * moduleDependencies[] = {
	"VuoMIDI.bc",
		"RtMidi",
		"CoreAudio.framework",
		"CoreMIDI.framework"
};

VuoMIDIIn nodeInstanceInit(
		VuoInputData(VuoMIDIDevice, {"default":{"isInput":true}}) device
)
{
	return VuoMIDIIn_make(device);
}

void nodeInstanceCallbackStart
(
		VuoInstanceData(VuoMIDIIn) context,
		VuoOutputTrigger(receivedNote, VuoMIDINote),
		VuoOutputTrigger(receivedController, VuoMIDIController)
//		VuoOutputTrigger(receivedAftertouch, VuoMIDIAftertouch),
//		VuoOutputTrigger(receivedCommand, VuoMIDICommand),
//		VuoOutputTrigger(receivedClock, VuoMIDIClock),
//		VuoOutputTrigger(receivedTimecode, VuoMIDITimecode),
//		VuoOutputTrigger(receivedSysEx, VuoMIDISysEx)
)
{
	VuoMIDIIn_enableTriggers(*context, receivedNote, receivedController);
}

void nodeInstanceEvent
(
		VuoInstanceData(VuoMIDIIn) context,
		VuoInputData(VuoMIDIDevice, {"default":{"isInput":true}}) device,
		VuoInputConductor(VuoPortNeverConducts, device) deviceEvent
)
{
	if (deviceEvent)
	{
		VuoMIDIIn_destroy(*context);
		*context = VuoMIDIIn_make(device);
	}
}

void nodeInstanceCallbackStop
(
		VuoInstanceData(VuoMIDIIn) context
)
{
	VuoMIDIIn_disableTriggers(*context);
}

void nodeInstanceFini
(
		VuoInstanceData(VuoMIDIIn) context
)
{
	VuoMIDIIn_destroy(*context);
}
