/**
 * @file
 * vuo.midi.send node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "VuoMIDI.h"

const char *moduleName = "Send MIDI Event";
const char *moduleDescription = "When an event is received on the `note` or `controller` ports, this node outputs the data to a MIDI device.  The first available MIDI output device is used, unless a specific `device` is provided.";
const char *moduleKeywords[] = { };
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = true;
/// @todo when https://b33p.net/kosada/node/4018 is fixed, move the VuoMIDI dependencies into VuoMIDI.cc.
const char * moduleDependencies[] = {
	"VuoMIDI.bc",
		"RtMidi",
		"CoreAudio.framework",
		"CoreMIDI.framework"
};

VuoMIDIOut nodeInstanceInit(
		VuoInputData(VuoMIDIDevice, {"default":{"isInput":false}}) device
)
{
	return VuoMIDIOut_make(device);
}

void nodeInstanceEvent
(
		VuoInstanceData(VuoMIDIOut) context,
		VuoInputData(VuoMIDIDevice, {"default":{"isInput":false}}) device,
		VuoInputConductor(VuoPortNeverConducts, device) deviceEvent,
		VuoInputData(VuoMIDINote,"") note,
		VuoInputConductor(VuoPortAlwaysConducts, note) noteEvent,
		VuoInputData(VuoMIDIController,"") controller,
		VuoInputConductor(VuoPortAlwaysConducts, controller) controllerEvent
//		VuoInputData(VuoMIDIAftertouch,"") aftertouch,
//		VuoInputData(VuoMIDICommand,"") command,
//		VuoInputData(VuoMIDIClock,"") clock,
//		VuoInputData(VuoMIDITimecode,"") timecode,
//		VuoInputData(VuoMIDISysEx,"") sysEx
)
{
	if (deviceEvent)
	{
		VuoMIDIOut_destroy(*context);
		*context = VuoMIDIOut_make(device);
	}

	if (noteEvent)
		VuoMIDIOut_sendNote(*context, note);
	if (controllerEvent)
		VuoMIDIOut_sendController(*context, controller);
}

void nodeInstanceFini
(
		VuoInstanceData(VuoMIDIOut) context
)
{
	VuoMIDIOut_destroy(*context);
}
