/**
 * @file
 * vuo.scene.object.make.image node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"

const char *moduleName = "Place Image in Scene";
const char *moduleDescription = "Creates a Scene Object from an image.  Specify `rotation` in degrees.  Specify the `width`; the height is calculated to preserve the image's aspect ratio.";
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = false;
const char *moduleKeywords[] = { "billboard", "sprite", "projector" };
const char *moduleDependencies[] = { };

void nodeEvent
(
		VuoInputData(VuoImage) image,
		VuoInputData(VuoPoint3d, {"default":{"x":0,"y":0,"z":0}}) center,
		VuoInputData(VuoPoint3d, {"default":{"x":0,"y":0,"z":0}}) rotation,
		VuoInputData(VuoReal, {"default":1}) width,
		VuoOutputData(VuoSceneObject) sceneObject
)
{
	if (!image)
	{
		*sceneObject = VuoSceneObject_valueFromString("");
		return;
	}

	VuoList_VuoVertices verticesList = VuoListCreate_VuoVertices();
	VuoListAppendValue_VuoVertices(verticesList, VuoVertices_getQuad());
	*sceneObject = VuoSceneObject_make(
				verticesList,
				VuoShader_makeImageShader(),
				VuoTransform_makeEuler(center, rotation, VuoPoint3d_make(width,image->pixelsHigh * width/image->pixelsWide,1)),
				VuoListCreate_VuoSceneObject()
			);
	VuoShader_addTexture(sceneObject->shader, image, "texture");
}
