/**
 * @file
 * vuo.scene.render.image node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include "VuoGlContext.h"
#include "VuoSceneRenderer.h"

#include <stdio.h>
#include <string.h>
#include <math.h>

#include <OpenGL/gl.h>
//#import <OpenGL/gl3.h>
/// @todo After we drop 10.6 support, switch back to gl3 and remove the below 4 lines.  See also r15430 for shader changes.
#include <OpenGL/glext.h>


const char *moduleName = "Render Scene to Image";
const char *moduleDescription = "Renders `rootSceneObject` and its descendants onto an image.";
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = false;
const char *moduleKeywords[] = { "draw", "opengl", "scenegraph", "graphics" };
/// @todo remove Qt dependencies after https://b33p.net/kosada/node/4018 is fixed
const char *moduleDependencies[] = {
	"VuoGlContext",
	"VuoSceneRenderer.bc",
};


struct nodeInstanceData
{
	VuoSceneRenderer *sceneRenderer;
};

struct nodeInstanceData *nodeInstanceInit(void)
{
	struct nodeInstanceData *context = (struct nodeInstanceData *)malloc(sizeof(struct nodeInstanceData));

	context->sceneRenderer = VuoSceneRenderer_create();

	vuoRegister(context, free);
	return context;
}

void nodeInstanceEvent
(
		VuoInstanceData(struct nodeInstanceData *) context,
		VuoInputData(VuoSceneObject) rootSceneObject,
		VuoInputData(VuoInteger, {"default":1024}) width,
		VuoInputData(VuoInteger, {"default":768}) height,
		VuoOutputData(VuoImage) image
)
{
	VuoSceneRenderer_setRootSceneObject((*context)->sceneRenderer, rootSceneObject);

	GLuint outputTexture;
	VuoGlContext_use();
	{
		VuoSceneRenderer_prepareContext((*context)->sceneRenderer);

		VuoSceneRenderer_regenerateProjectionMatrix((*context)->sceneRenderer, width, height);
		glViewport(0, 0, width, height);

		// Create a new GL Texture Object and Framebuffer Object.
		glGenTextures(1, &outputTexture);
		glBindTexture(GL_TEXTURE_2D, outputTexture);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);

		GLuint outputFramebuffer;
		glGenFramebuffers(1, &outputFramebuffer);
		glBindFramebuffer(GL_FRAMEBUFFER, outputFramebuffer);
		glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, outputTexture, 0);

		// Execute the shader every frame
		glBindFramebuffer(GL_FRAMEBUFFER, outputFramebuffer);
		{
			glClearColor(0,0,0,1);
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

			VuoSceneRenderer_draw((*context)->sceneRenderer);
		}
		glBindFramebuffer(GL_FRAMEBUFFER, 0);
		glDeleteFramebuffers(1, &outputFramebuffer);
	}
	VuoGlContext_disuse();

	*image = VuoImage_make(outputTexture, width, height);
}

void nodeInstanceFini
(
		VuoInstanceData(struct nodeInstanceData *) context
)
{
	VuoSceneRenderer_destroy((*context)->sceneRenderer);
}
