﻿/**
 * @file
 * vuo.text.append node implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include "node.h"
#include <stdlib.h>
#include <string.h>

const char *moduleName = "Append";
const char *moduleDescription = "Appends the input texts to create one composite text.";
const unsigned int moduleVersion = 100;
const bool nodeIsInterface = false;
const char *moduleKeywords[] = { "concatenate", "strcat" };
const char *moduleDependencies[] = { "c" };

void nodeEvent
(
		VuoInputData(VuoList_VuoText) texts,
		VuoOutputData(VuoText) compositeText
)
{
	unsigned long textsCount = VuoListGetCount_VuoText(texts);

	VuoText *textsArray = (VuoText *) malloc(textsCount * sizeof(VuoText));
	for (unsigned long i = 0; i < textsCount; ++i)
		textsArray[i] = VuoListGetValueAtIndex_VuoText(texts, i);

	*compositeText = VuoText_append(textsArray, textsCount);

	free(textsArray);
}
