/**
 * @file
 * VuoBoolean implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoBoolean.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "Boolean";
const char * moduleDescription = "A boolean (true/false) value.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "true", "false" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoBoolean
 * Parses the C string @c initializer to create a new value.
 */
VuoBoolean VuoBoolean_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoBoolean value = VuoBoolean_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoBoolean
 * Serializes @c value as a C string.
 */
char * VuoBoolean_stringFromValue(const VuoBoolean value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoBoolean_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoBoolean
 * Decodes the JSON object @c js, expected to contain a boolean, to create a new @c VuoBoolean.
 */
VuoBoolean VuoBoolean_valueFromJson(json_object * js)
{
	return json_object_get_boolean(js);
}

/**
 * @ingroup VuoBoolean
 * Encodes @c value as a JSON object.
 */
json_object * VuoBoolean_jsonFromValue(const VuoBoolean value)
{
	return json_object_new_boolean(value);
}

/**
 * @ingroup VuoBoolean
 * Same as @c %VuoBoolean_stringFromValue() — always shows the full value, since it's guaranteed to be pretty short.
 */
char * VuoBoolean_summaryFromValue(const VuoBoolean value)
{
	// Always show the full value, since it's guaranteed to be pretty short.
	return VuoBoolean_stringFromValue(value);
}
