/**
 * @file
 * VuoCurve implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoCurve.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "Curve";
const char * moduleDescription = "Type of curve.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "curve" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoCurve
 * Parses the C string @c initializer to create a new value.
 */
VuoCurve VuoCurve_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoCurve value = VuoCurve_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoCurve
 * Serializes @c value as a C string.
 */
char * VuoCurve_stringFromValue(const VuoCurve value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoCurve_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoCurve
 * Decodes the JSON object @c js, expected to contain a string, to create a new @c VuoCurve.
 */
VuoCurve VuoCurve_valueFromJson(json_object *js)
{
	const char *valueAsString = "";
	if (json_object_get_type(js) == json_type_string)
		valueAsString = json_object_get_string(js);

	VuoCurve curve = VuoCurve_Linear;

	if (strcmp(valueAsString, "linear") == 0) {
		curve = VuoCurve_Linear;
	} else if (strcmp(valueAsString, "sine") == 0) {
		curve = VuoCurve_Sine;
	} else if (strcmp(valueAsString, "gaussian") == 0) {
		curve = VuoCurve_Window_Gaussian;
	} else if (strcmp(valueAsString, "planck") == 0) {
		curve = VuoCurve_Window_Planck;
	} else if (strcmp(valueAsString, "kaiser") == 0) {
		curve = VuoCurve_Window_Kaiser;
	} else if (strcmp(valueAsString, "dolph-chebyshev") == 0) {
		curve = VuoCurve_Window_DolphChebyshev;
	} else if (strcmp(valueAsString, "poisson") == 0) {
		curve = VuoCurve_Window_Poisson;
	} else if (strcmp(valueAsString, "lanczos") == 0) {
		curve = VuoCurve_Window_Lanczos;
	} else if (strcmp(valueAsString, "parzen") == 0) {
		curve = VuoCurve_Window_Parzen;
	} else if (strcmp(valueAsString, "quadratic-in") == 0) {
		curve = VuoCurve_Quadratic_In;
	} else if (strcmp(valueAsString, "quadratic-out") == 0) {
		curve = VuoCurve_Quadratic_Out;
	} else if (strcmp(valueAsString, "quadratic-inout") == 0) {
		curve = VuoCurve_Quadratic_InOut;
	} else if (strcmp(valueAsString, "quadratic-outin") == 0) {
		curve = VuoCurve_Quadratic_OutIn;
	} else if (strcmp(valueAsString, "exponential-in") == 0) {
		curve = VuoCurve_Exponential_In;
	} else if (strcmp(valueAsString, "exponential-out") == 0) {
		curve = VuoCurve_Exponential_Out;
	} else if (strcmp(valueAsString, "exponential-inout") == 0) {
		curve = VuoCurve_Exponential_InOut;
	} else if (strcmp(valueAsString, "exponential-outin") == 0) {
		curve = VuoCurve_Exponential_OutIn;
	} else if (strcmp(valueAsString, "circular-in") == 0) {
		curve = VuoCurve_Circular_In;
	} else if (strcmp(valueAsString, "circular-out") == 0) {
		curve = VuoCurve_Circular_Out;
	} else if (strcmp(valueAsString, "circular-inout") == 0) {
		curve = VuoCurve_Circular_InOut;
	} else if (strcmp(valueAsString, "circular-outin") == 0) {
		curve = VuoCurve_Circular_OutIn;
	} else if (strcmp(valueAsString, "spring-in") == 0) {
		curve = VuoCurve_Spring_In;
	} else if (strcmp(valueAsString, "spring-out") == 0) {
		curve = VuoCurve_Spring_Out;
	} else if (strcmp(valueAsString, "spring-inout") == 0) {
		curve = VuoCurve_Spring_InOut;
	} else if (strcmp(valueAsString, "spring-outin") == 0) {
		curve = VuoCurve_Spring_OutIn;
	}

	return curve;
}

/**
 * @ingroup VuoCurve
 * Encodes @c value as a JSON object.
 */
json_object * VuoCurve_jsonFromValue(const VuoCurve value)
{
	char * valueAsString = "";

	switch (value) {
		case VuoCurve_Linear:
			valueAsString = "linear";
			break;
		case VuoCurve_Sine:
			valueAsString = "sine";
			break;
		case VuoCurve_Window_Gaussian:
			valueAsString = "gaussian";
			break;
		case VuoCurve_Window_Planck:
			valueAsString = "planck";
			break;
		case VuoCurve_Window_Kaiser:
			valueAsString = "kaiser";
			break;
		case VuoCurve_Window_DolphChebyshev:
			valueAsString = "dolph-chebyshev";
			break;
		case VuoCurve_Window_Poisson:
			valueAsString = "poisson";
			break;
		case VuoCurve_Window_Lanczos:
			valueAsString = "lanczos";
			break;
		case VuoCurve_Window_Parzen:
			valueAsString = "parzen";
			break;
		case VuoCurve_Quadratic_In:
			valueAsString = "quadratic-in";
			break;
		case VuoCurve_Quadratic_Out:
			valueAsString = "quadratic-out";
			break;
		case VuoCurve_Quadratic_InOut:
			valueAsString = "quadratic-inout";
			break;
		case VuoCurve_Quadratic_OutIn:
			valueAsString = "quadratic-outin";
			break;
		case VuoCurve_Exponential_In:
			valueAsString = "exponential-in";
			break;
		case VuoCurve_Exponential_Out:
			valueAsString = "exponential-out";
			break;
		case VuoCurve_Exponential_InOut:
			valueAsString = "exponential-inout";
			break;
		case VuoCurve_Exponential_OutIn:
			valueAsString = "exponential-outin";
			break;
		case VuoCurve_Circular_In:
			valueAsString = "circular-in";
			break;
		case VuoCurve_Circular_Out:
			valueAsString = "circular-out";
			break;
		case VuoCurve_Circular_InOut:
			valueAsString = "circular-inout";
			break;
		case VuoCurve_Circular_OutIn:
			valueAsString = "circular-outin";
			break;
		case VuoCurve_Spring_In:
			valueAsString = "spring-in";
			break;
		case VuoCurve_Spring_Out:
			valueAsString = "spring-out";
			break;
		case VuoCurve_Spring_InOut:
			valueAsString = "spring-inout";
			break;
		case VuoCurve_Spring_OutIn:
			valueAsString = "spring-outin";
			break;
	}

	return json_object_new_string(valueAsString);
}

/**
 * @ingroup VuoCurve
 * Same as @c %VuoCurve_stringFromValue()
 */
char * VuoCurve_summaryFromValue(const VuoCurve value)
{
	switch (value) {
		case VuoCurve_Linear:
			return strdup("Linear");
		case VuoCurve_Sine:
			return strdup("Sine");
		case VuoCurve_Window_Gaussian:
			return strdup("Gaussian");
		case VuoCurve_Window_Planck:
			return strdup("Planck");
		case VuoCurve_Window_Kaiser:
			return strdup("Kaiser");
		case VuoCurve_Window_DolphChebyshev:
			return strdup("Dolph-Chebyshev");
		case VuoCurve_Window_Poisson:
			return strdup("Poisson");
		case VuoCurve_Window_Lanczos:
			return strdup("Lanczos");
		case VuoCurve_Window_Parzen:
			return strdup("Parzen");
		case VuoCurve_Quadratic_In:
			return strdup("Quadratic-in");
		case VuoCurve_Quadratic_Out:
			return strdup("Quadratic-out");
		case VuoCurve_Quadratic_InOut:
			return strdup("Quadratic-inout");
		case VuoCurve_Quadratic_OutIn:
			return strdup("Quadratic-outin");
		case VuoCurve_Exponential_In:
			return strdup("Exponential-in");
		case VuoCurve_Exponential_Out:
			return strdup("Exponential-out");
		case VuoCurve_Exponential_InOut:
			return strdup("Exponential-inout");
		case VuoCurve_Exponential_OutIn:
			return strdup("Exponential-outin");
		case VuoCurve_Circular_In:
			return strdup("Circular-in");
		case VuoCurve_Circular_Out:
			return strdup("Circular-out");
		case VuoCurve_Circular_InOut:
			return strdup("Circular-inout");
		case VuoCurve_Circular_OutIn:
			return strdup("Circular-outin");
		case VuoCurve_Spring_In:
			return strdup("Spring-in");
		case VuoCurve_Spring_Out:
			return strdup("Spring-out");
		case VuoCurve_Spring_InOut:
			return strdup("Spring-inout");
		case VuoCurve_Spring_OutIn:
			return strdup("Spring-outin");
	}
}
