/**
 * @file
 * VuoCurveDomain implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoCurveDomain.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "Curve Domain";
const char * moduleDescription = "Domain that can be applied to a curve.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "curve", "domain" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoCurveDomain
 * Parses the C string @c initializer to create a new value.
 */
VuoCurveDomain VuoCurveDomain_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoCurveDomain value = VuoCurveDomain_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoCurveDomain
 * Serializes @c value as a C string.
 */
char * VuoCurveDomain_stringFromValue(const VuoCurveDomain value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoCurveDomain_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoCurveDomain
 * Decodes the JSON object @c js, expected to contain a string, to create a new @c VuoCurveDomain.
 */
VuoCurveDomain VuoCurveDomain_valueFromJson(json_object *js)
{
	const char *valueAsString = "";
	if (json_object_get_type(js) == json_type_string)
		valueAsString = json_object_get_string(js);

	VuoCurveDomain domain = VuoCurveDomain_Clamp;

	if (strcmp(valueAsString, "clamp") == 0) {
		domain = VuoCurveDomain_Clamp;
	} else if (strcmp(valueAsString, "infinite") == 0) {
		domain = VuoCurveDomain_Infinite;
	} else if (strcmp(valueAsString, "wrap") == 0) {
		domain = VuoCurveDomain_Wrap;
	} else if (strcmp(valueAsString, "mirror") == 0) {
		domain = VuoCurveDomain_Mirror;
	}

	return domain;
}

/**
 * @ingroup VuoCurveDomain
 * Encodes @c value as a JSON object.
 */
json_object * VuoCurveDomain_jsonFromValue(const VuoCurveDomain value)
{
	char * valueAsString = "";

	switch (value) {
		case VuoCurveDomain_Clamp:
			valueAsString = "clamp";
			break;
		case VuoCurveDomain_Infinite:
			valueAsString = "infinite";
			break;
		case VuoCurveDomain_Wrap:
			valueAsString = "wrap";
			break;
		case VuoCurveDomain_Mirror:
			valueAsString = "mirror";
			break;
	}

	return json_object_new_string(valueAsString);
}

/**
 * @ingroup VuoCurveDomain
 * Same as @c %VuoCurveDomain_stringFromValue()
 */
char * VuoCurveDomain_summaryFromValue(const VuoCurveDomain value)
{
	switch (value) {
		case VuoCurveDomain_Clamp:
			return strdup("Clamp");
		case VuoCurveDomain_Infinite:
			return strdup("Infinite");
		case VuoCurveDomain_Wrap:
			return strdup("Wrap");
		case VuoCurveDomain_Mirror:
			return strdup("Mirror");
	}
}
