/**
 * @file
 * VuoGradientNoise implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoGradientNoise.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "Gradient Noise";
const char * moduleDescription = "A method for generating gradient noise.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "perlin", "simplex" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoGradientNoise
 * Parses the C string @c initializer to create a new value.
 */
VuoGradientNoise VuoGradientNoise_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoGradientNoise value = VuoGradientNoise_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoGradientNoise
 * Serializes @c value as a C string.
 */
char * VuoGradientNoise_stringFromValue(const VuoGradientNoise value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoGradientNoise_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoGradientNoise
 * Decodes the JSON object @c js, expected to contain a string, to create a new @c VuoGradientNoise.
 */
VuoGradientNoise VuoGradientNoise_valueFromJson(json_object *js)
{
	const char *valueAsString = "";
	if (json_object_get_type(js) == json_type_string)
		valueAsString = json_object_get_string(js);

	VuoGradientNoise gn = VuoGradientNoise_Perlin;

	if (strcmp(valueAsString, "perlin") == 0) {
		gn = VuoGradientNoise_Perlin;
	} else if (strcmp(valueAsString, "simplex") == 0) {
		gn = VuoGradientNoise_Simplex;
	}

	return gn;
}

/**
 * @ingroup VuoGradientNoise
 * Encodes @c value as a JSON object.
 */
json_object * VuoGradientNoise_jsonFromValue(const VuoGradientNoise value)
{
	char * valueAsString = "";

	switch (value) {
		case VuoGradientNoise_Perlin:
			valueAsString = "perlin";
			break;
		case VuoGradientNoise_Simplex:
			valueAsString = "simplex";
			break;
	}

	return json_object_new_string(valueAsString);
}

/**
 * @ingroup VuoGradientNoise
 * Same as @c %VuoGradientnoise_stringFromValue()
 */
char * VuoGradientNoise_summaryFromValue(const VuoGradientNoise value)
{
	switch (value) {
		case VuoGradientNoise_Perlin:
		default:
			return strdup("Perlin");
		case VuoGradientNoise_Simplex:
			return strdup("Simplex");
	}
}
