/**
 * @file
 * VuoInteger implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoInteger.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "Integer";
const char * moduleDescription = "A signed 64-bit integer.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "number", "signed" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoInteger
 * Parses the C string @c initializer to create a new value.
 */
VuoInteger VuoInteger_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoInteger value = VuoInteger_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoInteger
 * Serializes @c value as a C string.
 */
char * VuoInteger_stringFromValue(const VuoInteger value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoInteger_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoInteger
 * Decodes the JSON object @c js, expected to contain a 64-bit integer, to create a new value.
 */
VuoInteger VuoInteger_valueFromJson(json_object * js)
{
	return json_object_get_int64(js);
}

/**
 * @ingroup VuoInteger
 * Encodes @c value as a JSON object.
 */
json_object * VuoInteger_jsonFromValue(const VuoInteger value)
{
	return json_object_new_int64(value);
}

/**
 * @ingroup VuoInteger
 * Same as @c %VuoInteger_stringFromValue() — always shows the full value, since it's guaranteed to be pretty short.
 */
char * VuoInteger_summaryFromValue(const VuoInteger value)
{
	// Always show the full value, since it's guaranteed to be pretty short.
	return VuoInteger_stringFromValue(value);
}
