/**
 * @file
 * VuoMIDIController implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoMIDIController.h"
#include "VuoInteger.h"
#include "VuoBoolean.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "MIDI Controller";
const char * moduleDescription = "A music controller event sent via MIDI.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "CC", "custom controller" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoMIDIController
 * Parses the C string @c initializer to create a new value.
 */
VuoMIDIController VuoMIDIController_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoMIDIController value = VuoMIDIController_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoMIDIController
 * Serializes @c value as a C string.
 */
char * VuoMIDIController_stringFromValue(const VuoMIDIController value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoMIDIController_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoMIDIController
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *   {
 *     "channel" : 1,
 *     "controllerNumber" : 1,
 *     "value" : 127
 *   }
 * }
 */
VuoMIDIController VuoMIDIController_valueFromJson(json_object * js)
{
	VuoMIDIController mn = {1,1,127};
	json_object *o = NULL;

	if (json_object_object_get_ex(js, "channel", &o))
		mn.channel = VuoInteger_valueFromJson(o);

	if (json_object_object_get_ex(js, "controllerNumber", &o))
		mn.controllerNumber = VuoInteger_valueFromJson(o);

	if (json_object_object_get_ex(js, "value", &o))
		mn.value = VuoInteger_valueFromJson(o);

	return mn;
}

/**
 * @ingroup VuoMIDIController
 * Encodes @c value as a JSON object.
 */
json_object * VuoMIDIController_jsonFromValue(const VuoMIDIController mn)
{
	json_object *js = json_object_new_object();

	json_object *channelObject = VuoInteger_jsonFromValue(mn.channel);
	json_object_object_add(js, "channel", channelObject);

	json_object *controllerNumberObject = VuoInteger_jsonFromValue(mn.controllerNumber);
	json_object_object_add(js, "controllerNumber", controllerNumberObject);

	json_object *valueObject = VuoInteger_jsonFromValue(mn.value);
	json_object_object_add(js, "value", valueObject);

	return js;
}

/**
 * @ingroup VuoMIDIController
 * Returns a compact string representation of @c value (comma-separated coordinates).
 */
char * VuoMIDIController_summaryFromValue(const VuoMIDIController mn)
{
	const char *format = "Channel %d, controller %d (0x%02x): value %d";

	int size = snprintf(NULL,0,format,mn.channel,mn.controllerNumber,mn.controllerNumber,mn.value);
	char *valueAsString = (char *)malloc(size+1);
	snprintf(valueAsString,size+1,format,mn.channel,mn.controllerNumber,mn.controllerNumber,mn.value);

	return valueAsString;
}
