/**
 * @file
 * VuoMIDIDevice C type definition.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#ifndef VUOMIDIDEVICE_H
#define VUOMIDIDEVICE_H

/**
 * @ingroup VuoTypes
 * @defgroup VuoMIDIDevice VuoMIDIDevice
 * A set of specifications for choosing a MIDI device.
 *
 * @{
 */

#include "VuoInteger.h"
#include "VuoText.h"
#include "VuoBoolean.h"

/**
 * A set of specifications for choosing a MIDI device.
 */
typedef struct
{
	VuoInteger id;	///< If @c id is non-negative, use the specified device identifier.
	VuoText name;	///< If @c id is negative, use the first device whose name contains @c name.
	VuoBoolean isInput;	///< Is this an input or output device?
} VuoMIDIDevice;

VuoMIDIDevice VuoMIDIDevice_valueFromString(const char * initializer);
char * VuoMIDIDevice_stringFromValue(const VuoMIDIDevice value);
VuoMIDIDevice VuoMIDIDevice_valueFromJson(struct json_object * js);
struct json_object * VuoMIDIDevice_jsonFromValue(const VuoMIDIDevice value);
char * VuoMIDIDevice_summaryFromValue(const VuoMIDIDevice value);
void VuoMIDIDevice_retain(VuoMIDIDevice value);
void VuoMIDIDevice_release(VuoMIDIDevice value);

/**
 * Returns a note event with the specified values.
 */
static inline VuoMIDIDevice VuoMIDIDevice_make(VuoInteger id, VuoText name, VuoBoolean isInput) __attribute__((const));
static inline VuoMIDIDevice VuoMIDIDevice_make(VuoInteger id, VuoText name, VuoBoolean isInput)
{
	VuoMIDIDevice md = {id,name,isInput};
	return md;
}

/**
 * @}
 */

#endif
