/**
 * @file
 * VuoNoise implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoNoise.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "Noise";
const char * moduleDescription = "Type of noise.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "noise" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoNoise
 * Parses the C string @c initializer to create a new value.
 */
VuoNoise VuoNoise_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoNoise value = VuoNoise_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoNoise
 * Serializes @c value as a C string.
 */
char * VuoNoise_stringFromValue(const VuoNoise value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoNoise_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoNoise
 * Decodes the JSON object @c js, expected to contain a string, to create a new @c VuoNoise.
 */
VuoNoise VuoNoise_valueFromJson(json_object *js)
{
	const char *valueAsString = "";
	if (json_object_get_type(js) == json_type_string)
		valueAsString = json_object_get_string(js);

	VuoNoise noise = VuoNoise_White;

	if (strcmp(valueAsString, "white") == 0) {
		noise = VuoNoise_White;
	} else if (strcmp(valueAsString, "grey") == 0) {
		noise = VuoNoise_Grey;
	} else if (strcmp(valueAsString, "pink") == 0) {
		noise = VuoNoise_Pink;
	} else if (strcmp(valueAsString, "brown") == 0) {
		noise = VuoNoise_Brown;
	} else if (strcmp(valueAsString, "blue") == 0) {
		noise = VuoNoise_Blue;
	} else if (strcmp(valueAsString, "violet") == 0) {
		noise = VuoNoise_Violet;
	}

	return noise;
}

/**
 * @ingroup VuoNoise
 * Encodes @c value as a JSON object.
 */
json_object * VuoNoise_jsonFromValue(const VuoNoise value)
{
	char * valueAsString = "";

	switch (value) {
		case VuoNoise_White:
			valueAsString = "white";
			break;
		case VuoNoise_Grey:
			valueAsString = "grey";
			break;
		case VuoNoise_Pink:
			valueAsString = "pink";
			break;
		case VuoNoise_Brown:
			valueAsString = "brown";
			break;
		case VuoNoise_Blue:
			valueAsString = "blue";
			break;
		case VuoNoise_Violet:
			valueAsString = "violet";
			break;
	}

	return json_object_new_string(valueAsString);
}

/**
 * @ingroup VuoNoise
 * Same as @c %VuoNoise_stringFromValue()
 */
char * VuoNoise_summaryFromValue(const VuoNoise value)
{
	switch (value) {
		case VuoNoise_White:
			return strdup("White");
		case VuoNoise_Grey:
			return strdup("Grey");
		case VuoNoise_Pink:
			return strdup("Pink");
		case VuoNoise_Brown:
			return strdup("Brown");
		case VuoNoise_Blue:
			return strdup("Blue");
		case VuoNoise_Violet:
			return strdup("Violet");
	}
}
