/**
 * @file
 * VuoPoint3d implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoPoint3d.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "3D Point";
const char * moduleDescription = "A floating-point 3-dimensional Cartesian spatial location.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "coordinate" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoPoint3d
 * Parses the C string @c initializer to create a new value.
 */
VuoPoint3d VuoPoint3d_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoPoint3d value = VuoPoint3d_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoPoint3d
 * Serializes @c value as a C string.
 */
char * VuoPoint3d_stringFromValue(const VuoPoint3d value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoPoint3d_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoPoint3d
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *   {
 *     "x" : 0.5,
 *     "y" : 1,
 *     "z" : 0
 *   }
 * }
 */
VuoPoint3d VuoPoint3d_valueFromJson(json_object * js)
{
	VuoPoint3d point = {0,0,0};
	json_object *o = NULL;

	if (json_object_object_get_ex(js, "x", &o))
		point.x = json_object_get_double(o);

	if (json_object_object_get_ex(js, "y", &o))
		point.y = json_object_get_double(o);

	if (json_object_object_get_ex(js, "z", &o))
		point.z = json_object_get_double(o);

	return point;
}

/**
 * @ingroup VuoPoint2d
 * Encodes @c value as a JSON object.
 */
json_object * VuoPoint3d_jsonFromValue(const VuoPoint3d value)
{
	json_object *js = json_object_new_object();

	json_object *xObject = json_object_new_double(value.x);
	json_object_object_add(js, "x", xObject);

	json_object *yObject = json_object_new_double(value.x);
	json_object_object_add(js, "y", yObject);

	json_object *zObject = json_object_new_double(value.z);
	json_object_object_add(js, "z", zObject);

	return js;
}

/**
 * @ingroup VuoPoint3d
 * Returns a compact string representation of @c value (comma-separated coordinates).
 */
char * VuoPoint3d_summaryFromValue(const VuoPoint3d value)
{
	const char *format = "%g,%g,%g";
	int size = snprintf(NULL,0,format,value.x,value.y,value.z);
	char *valueAsString = (char *)malloc(size+1);
	snprintf(valueAsString,size+1,format,value.x,value.y,value.z);
	return valueAsString;
}
