/**
 * @file
 * VuoPoint4d implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoPoint4d.h"
#include "VuoReal.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "4D Point";
const char * moduleDescription = "A floating-point 4-dimensional point.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "coordinate" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoPoint4d
 * Parses the C string @c initializer to create a new value.
 */
VuoPoint4d VuoPoint4d_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoPoint4d value = VuoPoint4d_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoPoint4d
 * Serializes @c value as a C string.
 */
char * VuoPoint4d_stringFromValue(const VuoPoint4d value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoPoint4d_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoPoint3d
 * Decodes the JSON object @c js to create a new value.
 *
 * @eg{
 *   {
 *     "x" : 0.5,
 *     "y" : 1,
 *     "z" : 0,
 *     "w" : 0
 *   }
 * }
 */
VuoPoint4d VuoPoint4d_valueFromJson(json_object * js)
{
	VuoPoint4d point = {0,0,0,0};
	json_object *o = NULL;

	if (json_object_object_get_ex(js, "x", &o))
		point.x = VuoReal_valueFromJson(o);

	if (json_object_object_get_ex(js, "y", &o))
		point.y = VuoReal_valueFromJson(o);

	if (json_object_object_get_ex(js, "z", &o))
		point.z = VuoReal_valueFromJson(o);

	if (json_object_object_get_ex(js, "w", &o))
		point.w = VuoReal_valueFromJson(o);

	return point;
}

/**
 * @ingroup VuoPoint2d
 * Encodes @c value as a JSON object.
 */
json_object * VuoPoint4d_jsonFromValue(const VuoPoint4d value)
{
	json_object *js = json_object_new_object();

	json_object *xObject = VuoReal_jsonFromValue(value.x);
	json_object_object_add(js, "x", xObject);

	json_object *yObject = VuoReal_jsonFromValue(value.x);
	json_object_object_add(js, "y", yObject);

	json_object *zObject = VuoReal_jsonFromValue(value.z);
	json_object_object_add(js, "z", zObject);

	json_object *wObject = VuoReal_jsonFromValue(value.w);
	json_object_object_add(js, "w", wObject);

	return js;
}

/**
 * @ingroup VuoPoint4d
 * Returns a compact string representation of @c value (comma-separated coordinates).
 */
char * VuoPoint4d_summaryFromValue(const VuoPoint4d value)
{
	const char *format = "%g,%g,%g,%g";
	int size = snprintf(NULL,0,format,value.x,value.y,value.z,value.w);
	char *valueAsString = (char *)malloc(size+1);
	snprintf(valueAsString,size+1,format,value.x,value.y,value.z,value.w);
	return valueAsString;
}
