/**
 * @file
 * VuoReal implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoReal.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "Real";
const char * moduleDescription = "A floating-point number.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "double", "float", "number" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoReal
 * Parses the C string @c initializer to create a new value.
 */
VuoReal VuoReal_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoReal value = VuoReal_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoReal
 * Serializes @c value as a C string.
 */
char * VuoReal_stringFromValue(const VuoReal value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoReal_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoSceneObject
 * Decodes the JSON object @c js, expected to contain a double, to create a new value.
 */
VuoReal VuoReal_valueFromJson(json_object * js)
{
	return json_object_get_double(js);
}

/**
 * @ingroup VuoReal
 * Encodes @c value as a JSON object.
 */
json_object * VuoReal_jsonFromValue(const VuoReal value)
{
	return json_object_new_double(value);
}

/**
 * @ingroup VuoReal
 * Returns a string representation of @c value (either decimal or scientific notation, whichever is shorter).
 */
char * VuoReal_summaryFromValue(const VuoReal value)
{
	const char *format = "%g";
	int size = snprintf(NULL,0,format,value);
	char *valueAsString = (char *)malloc(size+1);
	snprintf(valueAsString,size+1,format,value);
	return valueAsString;
}
