/**
 * @file
 * VuoWave implementation.
 *
 * @copyright Copyright © 2012–2013 Kosada Incorporated.
 * This code may be modified and distributed under the terms of the MIT License.
 * For more information, see http://vuo.org/license.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "type.h"
#include "VuoWave.h"

/// @{
#ifdef VUO_COMPILER
const char * moduleName = "Wave";
const char * moduleDescription = "Type of wave.";
const unsigned int moduleVersion = 100;
const char * moduleKeywords[] = { "wave" };
const char * moduleDependencies[] = { "c" };
#endif
/// @}

/**
 * @ingroup VuoWave
 * Parses the C string @c initializer to create a new value.
 */
VuoWave VuoWave_valueFromString(const char * initializer)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object * js = json_tokener_parse(initializer);
	VuoWave value = VuoWave_valueFromJson(js);
	json_object_put(js);
	return value;
}

/**
 * @ingroup VuoWave
 * Serializes @c value as a C string.
 */
char * VuoWave_stringFromValue(const VuoWave value)
{
	/// @todo This function body will be generated by vuo-compile (https://b33p.net/kosada/node/5521)
	json_object *js = VuoWave_jsonFromValue(value);
	char *string = strdup(json_object_to_json_string_ext(js,JSON_C_TO_STRING_PLAIN));
	json_object_put(js);
	return string;
}

/**
 * @ingroup VuoWave
 * Decodes the JSON object @c js, expected to contain a string, to create a new @c VuoWave.
 */
VuoWave VuoWave_valueFromJson(json_object *js)
{
	const char *valueAsString = "";
	if (json_object_get_type(js) == json_type_string)
		valueAsString = json_object_get_string(js);

	VuoWave wave = VuoWave_Sine;

	if (! strcmp(valueAsString, "sine"))
		wave = VuoWave_Sine;
	else if (! strcmp(valueAsString, "triangle"))
		wave = VuoWave_Triangle;
	else if (! strcmp(valueAsString, "sawtooth"))
		wave = VuoWave_Sawtooth;

	return wave;
}

/**
 * @ingroup VuoWave
 * Encodes @c value as a JSON object.
 */
json_object * VuoWave_jsonFromValue(const VuoWave value)
{
	char *valueAsString = "";

	switch (value) {
		case VuoWave_Sine:
			valueAsString = "sine";
			break;
		case VuoWave_Triangle:
			valueAsString = "triangle";
			break;
		case VuoWave_Sawtooth:
			valueAsString = "sawtooth";
			break;
	}

	return json_object_new_string(valueAsString);
}

/**
 * @ingroup VuoWave
 * Same as @c %VuoWave_stringFromValue()
 */
char * VuoWave_summaryFromValue(const VuoWave value)
{
	switch (value) {
		case VuoWave_Sine:
			return strdup("Sine");
		case VuoWave_Triangle:
			return strdup("Triangle");
		case VuoWave_Sawtooth:
			return strdup("Sawtooth");
	}
}
