#!/bin/bash

if [ $# -lt 1 ]; then
	echo -e "Generates type-specific list headers and implementations for the specified types."
	echo -e ""
	echo -e "Usage:"
	echo -e "\t$0 VuoTypeName:isReferenceCounted VuoTypeName:isReferenceCounted ..."
	echo -e ""
	echo -e "Prints the names of the created source file(s)."
	exit 1
fi

for kv in $* ; do
	vuoTypeName="${kv%%:*}"
	isReferenceCounted="${kv##*:}"
	typeSpecificH="VuoList_$vuoTypeName.h"
	typeSpecificCC="VuoList_$vuoTypeName.cc"

	if [ ! -f $typeSpecificH -o VuoList.h -nt $typeSpecificH ]; then
		[ -f $typeSpecificH ] && chmod +w $typeSpecificH
		cat VuoList.h \
			| sed "s/#error.*/\/\/ This header is generated by vuo\/type\/list\/generateVariants.sh./" \
			| sed "s/LIST_TYPE/VuoList_$vuoTypeName/g" \
			| sed "s/ELEMENT_TYPE/$vuoTypeName/g" \
			> $typeSpecificH
	fi

	if [ ! -f $typeSpecificCC -o VuoList.cc -nt $typeSpecificCC ]; then
		[ -f $typeSpecificCC ] && chmod +w $typeSpecificCC
		cat VuoList.cc \
			| sed "s/#error.*/\/\/ This module is generated by vuo\/type\/list\/generateVariants.sh./" \
			| sed "s/LIST_TYPE/VuoList_$vuoTypeName/g" \
			| sed "s/ELEMENT_TYPE/$vuoTypeName/g" \
			| sed "s/IS_ELEMENT_REFERENCE_COUNTED/$isReferenceCounted/g" \
			> $typeSpecificCC
	fi

	chmod -w $typeSpecificH $typeSpecificCC

	echo "VuoList_$vuoTypeName.cc"
done
